{% load static %}
<script type="text/javascript" src="{% static "js/testrun_actions.js" %}"></script>
<script type="text/javascript" src="{% static "js/management_actions.js" %}"></script>
<script type="text/javascript">
	Nitrate.Utils.after_page_load(Nitrate.TestRuns.AdvancedSearch.on_load);
</script>
{% include "run/common/run_advance_filtered.html" %}
