{% load static %}
<div id="run-statistics" class="statu" style="float:left;" >
	<div class="progress-bar" style="float:none">
		<div class="percent"><span id='complete_percent'>{{ status_stats.CompletedPercentage|floatformat:0 }}</span>%</div>
		<div class="progress-inner" style="width:{{ status_stats.CompletedPercentage|floatformat:"0" }}%;">
			<div class="progress-failed" style="width:{{ status_stats.FailurePercentage|floatformat:"0" }}%;"></div>
		</div>
	</div>
	<ul>
		{% for status_pk, total_info in status_stats.StatusSubtotal.items %}
		<li>
			<label>{{ total_info.1.name }}</label>
			{% ifnotequal total_info.0 0 %}
			<span id='{{ total_info.1.name }}'>[<a href="javascript:void(0)" class="js-status-subtotal" data-param="{{ total_info.1.pk }}">{{ total_info.0 }}</a>]</span>
			{% else %}
			<span id='{{ total_info.1.name }}'>[<a>{{ total_info.0 }}</a>]</span>
			{% endifnotequal %}
		</li>
		{% endfor %}
		<li>
			<label>TOTAL</label>
			<span id="TOTAL">[</span><a href="javascript:void(0)" class="js-caserun-total">{{ status_stats.CaseRunsTotalCount }}</a><span>]</span>
		</li>
	</ul>
	<div class="clear"></div>
	<ul>
		<li style="background:url({% static "images/btn_bg_report.png" %}) no-repeat; height:25px; line-height:25px; width:57px; padding-left:30px; margin-right:15px;"><a href="{% url "run-report" test_run.run_id %}" title="report of test caserun">Report</a></li>
		<li style="background:url({% static "images/btn_bg_showbug.png" %}) no-repeat; height:25px; line-height:25px; width:133px; padding-left:30px;">
		{% ifequal test_case_run_issues_count 0 %}
			<span id="total_run_issues_count">No Issues</span>
		{% else %}
			<span id="total_run_issues_count"><a href="{% url "run-report" test_run.run_id %}#issues" title="Show All Issues">Issues [{{ test_case_run_issues_count }}]</a></span>
		{% endifequal %}
		</li>
	</ul>
</div>
