{% extends "tcms_base.html" %}

{% load extra_filters %}
{% load static %}

{% block subtitle %}Test cases{% endblock %}

{% block custom_stylesheet %}
<link rel="stylesheet" type="text/css" href="{% static "style/dataTables/jquery.dataTables.css" %}" />
<link rel="stylesheet" type="text/css" href="{% static "style/dataTables/jquery.dataTables_themeroller.css" %}" />
{% endblock %}
{% block custom_javascript %}
<script type="text/javascript" src="{% static "js/lib/jquery.dataTables.js" %}"></script>
<script type="text/javascript" src="{% static "js/testplan_actions.js" %}"></script>
<script type="text/javascript" src="{% static "js/testcase_actions.js" %}"></script>
<script type="text/javascript">
	Nitrate.TestCases.List.Param = {
		'case_printable': '{% url "cases-printable" %}',
		'case_export':'{% url "cases-export" %}'
	};
Nitrate.Utils.after_page_load(Nitrate.TestCases.List.on_load);
</script>
{% endblock %}
{% block contents %}
<div id="content">
	<div class="sprites crumble">
		<a href="{% url "nitrate-index" %}">Home</a> >>
		{% if query_result %}
		<a href="{% url "cases-search" %}">Search Cases</a> >> Search result
		{% else %}
		Search Cases
		{% endif %}
	</div>
	<form action="{% url "cases-search" %}" method="get">
		<div id="itemSearch" class="itemSearch">
			<h2>Search Case</h2>
			{% include "case/form/search.html" %}
			<div class='clear'></div>
			<div class="errors">{{ search_form.errors }}</div>
			<div class="bottom" >
				<input type="submit" size="7" name="a" value="search" name="" title="Search test case" class="left_float" />
				<input type="reset" size="7" value="Reset" name="" title="Reset the search keyword" class="left_float" />
				<div class="allSearch">
					<a href="/advance-search/">Advanced Search</a>
				</div>
			</div>
		</div>
	</form>
	<!--case list-->
    {% if search_criterias %}
	    {% include "case/common/case_filtered.html" %}
    {% endif %}
</div>
{% endblock %}
