{% load static %}
<script type="text/javascript" src="{% static "js/testcase_actions.js" %}"></script>
<script type="text/javascript" language="javascript" charset="utf-8">
	Nitrate.TestCases.List.Param = {
		'case_printable': '{% url "cases-printable" %}',
		'case_export':'{% url "cases-export" %}'
	};
	Nitrate.Utils.after_page_load(Nitrate.TestCases.AdvanceList.on_load);
</script>
{% include "case/common/case_advance_filtered.html" %}
